using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinDataSource;
using System.Collections;
using Infragistics.Win;
using System.Threading;

namespace WinGridPaintThrottling
{
	public partial class Form1 : Form
	{
		public Form1( )
		{
			InitializeComponent( );
		}

		// This will paint 4 times a second.
		private const int PAINT_INTERVAL = 250; // milliseconds

		// Amount of time to restore the color after. Should be multiple of PAINT_INTERVAL
		// since we are using the same paint timer.
		private const int RESTORE_COLOR_TIME = 250; // milliseconds

		// Paint timer is used for repainting the grid at specifi intervals.
		private System.Windows.Forms.Timer paintTimer = new System.Windows.Forms.Timer( );

		// Thread where cell updates are coming.
		private Thread cellUpdateThread;	

		// For creating random updates.
		private Random r = new Random( );

		// Keeps track of cells that had their color set as a result of value change
		private Dictionary<UltraGridCell, DateTime> updatedCellsTable = new Dictionary<UltraGridCell, DateTime>( );

		// For keeping track of statistics on how many cells got updated etc...
		private int totalNumberOfCellUpdates = 0;
		private DateTime updateStartTime = DateTime.Now;

		// These are the enqued cell updates that need to be reflected in the grid.
		private object syncObj = new object( );
		private List<CellUpdate> cellUpdates = new List<CellUpdate>( );

		private void Form1_Load( object sender, EventArgs e )
		{
			UltraDataSource uds = new UltraDataSource( );

			for ( int i = 0; i < 20; i++ )
				uds.Band.Columns.Add( "Col" + i, typeof( double ) );

			uds.Rows.SetCount( 100 );
			this.ultraGrid1.SetDataBinding( uds, null );

			// Setup paint timer
			this.paintTimer.Interval = PAINT_INTERVAL;
			this.paintTimer.Tick += new EventHandler( paintTimer_Tick );
			this.paintTimer.Enabled = true;

			this.cellUpdateThread = new Thread( new ThreadStart( this.EnqueCellUpdates_Threaded ) );
			this.cellUpdateThread.Start( );
		}

		private void ultraGrid1_InitializeLayout( object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e )
		{
			// Setting the CellDisplayStyle can have significant impact on the rendering performance of the
			// UltraGrid. This property indicates whether to embedd an embeddable ui element inside each cell
			// or to render the value directly. The bahavior is typically the same unless you are using creation
			// filter or draw filter to manipulate embeddable ui element hierarchy. Note that when editing a
			// cell, the embeddable ui element will be used and therefore the editing behavior will be the same.
			//
			foreach ( UltraGridBand band in e.Layout.Bands )
			{
				foreach ( UltraGridColumn column in band.Columns )
				{
					// For string data types, typically formatting or applying editor data filters is not
					// necessary. Typically you simply want to draw the underlying cell value as it is.
					// For other types, you may want to apply formatting and if you are using editor data
					// filter, apply the data filter as well. Use FormattedText for those cases.
					//
					if ( typeof( string ) == column.DataType )
						column.CellDisplayStyle = CellDisplayStyle.PlainText;
					else
						column.CellDisplayStyle = CellDisplayStyle.FormattedText;
				}
			}

			// If your grid is using the same back color for the cells as rows (which are the default
			// UltraGrid settings), then you can prevent cells from drawing the background color
			// since the rows are going to draw it. Setting the CellAppearance's BackColor to
			// Transparent will prevent them from drawing any backround.
			//
			e.Layout.Override.CellAppearance.BackColor = Color.Transparent;

			// You can take this one step further. If the UltraGrid's background color is also
			// the same as the row backcolor then you can prevent the rows from drawing the
			// back color.
			//
			e.Layout.Override.RowAppearance.BackColor = Color.Transparent;

			// By default the UltraGrid syncrhonizes its active row with the currency manager so other
			// controls bound to the same datasource can syncrhonize their active row (current item)
			// with the UltraGrid's active row and also the other way around. Your application may not
			// be making any use of such currency manager syncrhonization. For example, you may have only
			// one grid bound to the same datasource and thus there is no need to syncrhonize with the
			// currency manager. In that case you can disable this feature. This typically results in
			// faster row navigation using up/down arrow key, especially in hierarchical situation.
			//
			this.ultraGrid1.SyncWithCurrencyManager = false;

			// By default the UltraGrid repaints cells as the mouse enters or leaves the cells.
			// It does this so custom hot-track cell drawing can be easily accomplished using
			// a ui element creation filter or draw filter. Typically this does not pose a
			// performance problem however you may still want to disable this behavior.
			//
			e.Layout.CellHottrackInvalidationStyle = CellHottrackInvalidationStyle.OnlyIfHottrackAppearance;

			// You can try different scrolling related settings to alleviate perceived 
			// scrolling jerkiness.
			//this.ultraGrid1.DisplayLayout.PriorityScrolling = true;
			this.ultraGrid1.DisplayLayout.UseScrollWindow = UseScrollWindow.None;
			//this.ultraGrid1.DisplayLayout.ScrollStyle = ScrollStyle.Deferred;
		}


		// Class for storing each cell update.
		private class CellUpdate
		{
			internal UltraDataRow dataRow;
			internal int colIndex;
			internal object value;

			internal CellUpdate( UltraDataRow dataRow, int colIndex, object value )
			{
				this.dataRow = dataRow;
				this.colIndex = colIndex;
				this.value = value;
			}
		}
	

		// This runs on separate thread and queues up cell updates.
		private void EnqueCellUpdates_Threaded( )
		{
			UltraDataSource uds = this.ultraGrid1.DataSource as UltraDataSource;
			while ( true )
			{
				lock ( this.syncObj )
				{
					for ( int i = 0; i < 50; i++ )
					{
						int rowIndex = r.Next( uds.Rows.Count );
						int colIndex = r.Next( uds.Band.Columns.Count );
						object newValue = r.Next( 10000 );

						this.cellUpdates.Add( new CellUpdate( uds.Rows[rowIndex], colIndex, newValue ) );
					}
				}

				Thread.Sleep( 20 );
			}
		}

		// This is for painting the grid specific number of times a second. It also
		// applies pending cell updates to the grid as well as manages color coding
		// of changed cells.
		private void paintTimer_Tick( object sender, EventArgs e )
		{
			this.ultraGrid1.BeginUpdate( );
			try
			{
				this.RestoreOriginalColor( );
				this.ProcessPendingUpdates( );
			}
			finally
			{
				TimeSpan totalTime = DateTime.Now - updateStartTime;
				this.Text = string.Format( "Total updates = {0}, average updates = {1}/sec",
					this.totalNumberOfCellUpdates, 
					(int)( this.totalNumberOfCellUpdates / totalTime.TotalSeconds ) );

				bool invalidate = true;
				this.ultraGrid1.EndUpdate( invalidate );
			}
		}

		private void ProcessPendingUpdates( )
		{
			List<CellUpdate> updateList;

			lock ( this.syncObj )
			{
				updateList = this.cellUpdates;
				this.cellUpdates = new List<CellUpdate>( );
			}

			RowsCollection gridRows = this.ultraGrid1.Rows;

			// Suspend/Resume BindingNotifications calls are specific to UltraDataSource 
			// only. They are not necessary however they add to efficiency.
			UltraDataSource uds = this.ultraGrid1.DataSource as UltraDataSource;
			if ( null != uds )
				uds.SuspendBindingNotifications( );
			try
			{
				DateTime now = DateTime.Now;

				foreach ( CellUpdate ii in updateList )
				{
					int listIndex = ii.dataRow.Index;
					if ( listIndex >= 0 )
					{
						UltraGridRow row = gridRows.GetRowWithListIndex( listIndex );
						UltraGridCell cell = row.Cells[ii.colIndex];
						cell.Value = ii.value;

						cell.Appearance.BackColor = Color.LightBlue;

						this.updatedCellsTable[cell] = now;
						this.totalNumberOfCellUpdates++;
					}
				}
			}
			finally
			{
				if ( null != uds )
					uds.ResumeBindingNotifications( );
			}
		}

		/// <summary>
		/// Restores appearances on cells that were updated and color coded more than 
		/// RESTORE_COLOR_TIME time ago.
		/// </summary>
		private void RestoreOriginalColor( )
		{
			DateTime now = DateTime.Now;
			List<UltraGridCell> removeCells = new List<UltraGridCell>( );

			foreach ( KeyValuePair<UltraGridCell, DateTime> entry in this.updatedCellsTable )
			{
				TimeSpan span = now - entry.Value;

				if ( span.TotalMilliseconds > RESTORE_COLOR_TIME )
				{
					UltraGridCell cell = entry.Key;
					cell.Appearance.Reset( );
					removeCells.Add( cell );
				}
			}

			// Remove cells from the table whose appearances were restored.
			foreach ( UltraGridCell cell in removeCells )
				this.updatedCellsTable.Remove( cell );
		}

		private void Form1_FormClosing( object sender, FormClosingEventArgs e )
		{
			this.cellUpdateThread.Abort( );
		}


	}
}